@@ -105,6 +105,10 @@ def do_metric_create_raw(mc, args):
            'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
            'that confuse the CLI parser.',
            action='append')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
 def do_metric_list(mc, args):
     '''List metrics for this tenant.'''
     fields = {}
@@ -112,6 +116,11 @@ def do_metric_list(mc, args):
         fields['name'] = args.name
     if args.dimensions:
         fields['dimensions'] = utils.format_parameters(args.dimensions)
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+
     try:
         metric = mc.metrics.list(**fields)
     except exc.HTTPException as he:
@@ -143,48 +152,46 @@ def do_metric_list(mc, args):
 
 def format_measure_id(measurements):
     # returns newline separated measurements id's for the id column
-    meas_string_list = list()
-    for meas in measurements:
-        # meas_string = '{:10d}'.format(meas[0])
-        meas_string = '{:20d}'.format(meas[0])
-        meas_string_list.append(meas_string)
-    return '\n'.join(meas_string_list)
+    measure_string_list = list()
+    for measure in measurements:
+        measure_string_list.append('{}'.format(measure[0]))
+    return '\n'.join(measure_string_list)
 
 
 def format_measure_timestamp(measurements):
     # returns newline separated times for the timestamp column
-    meas_string_list = list()
-    for meas in measurements:
-        meas_string_list.append(str(meas[1]))
-    return '\n'.join(meas_string_list)
+    measure_string_list = list()
+    for measure in measurements:
+        measure_string_list.append(str(measure[1]))
+    return '\n'.join(measure_string_list)
 
 
 def format_value_meta(measurements):
     # returns newline separated values for the value column
-    meas_string_list = list()
-    for meas in measurements:
-        if len(meas) < 4:
-            meas_string = ""
+    measure_string_list = list()
+    for measure in measurements:
+        if len(measure) < 4:
+            measure_string = ""
         else:
             meta_string_list = []
-            for k, v in meas[3].items():
+            for k, v in measure[3].items():
                 if isinstance(v, numbers.Number):
                     m_str = k + ': ' + str(v)
                 else:
                     m_str = k + ': ' + v
                 meta_string_list.append(m_str)
-            meas_string = ','.join(meta_string_list)
-        meas_string_list.append(meas_string)
-    return '\n'.join(meas_string_list)
+            measure_string = ','.join(meta_string_list)
+        measure_string_list.append(measure_string)
+    return '\n'.join(measure_string_list)
 
 
 def format_measure_value(measurements):
     # returns newline separated values for the value column
-    meas_string_list = list()
-    for meas in measurements:
-        meas_string = '{:12.2f}'.format(meas[2])
-        meas_string_list.append(meas_string)
-    return '\n'.join(meas_string_list)
+    measure_string_list = list()
+    for measure in measurements:
+        measure_string = '{:12.2f}'.format(measure[2])
+        measure_string_list.append(measure_string)
+    return '\n'.join(measure_string_list)
 
 
 def format_statistic_timestamp(statistics, columns, name):
@@ -241,7 +248,7 @@ def format_metric_dimensions(metrics):
     return '\n'.join(metric_string_list)
 
 
-@utils.arg('name', metavar='<METRIC_NAME>',
+@utils.arg('--name', metavar='<METRIC_NAME>',
            help='Name of the metric to list measurements.')
 @utils.arg('--dimensions', metavar='<KEY1=VALUE1,KEY2=VALUE2...>',
            help='key value pair used to specify a metric dimension. '
@@ -254,15 +261,30 @@ def format_metric_dimensions(metrics):
            help='measurements >= UTC time. format: 2014-01-01T00:00:00Z.')
 @utils.arg('--endtime', metavar='<UTC_END_TIME>',
            help='measurements <= UTC time. format: 2014-01-01T00:00:00Z.')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
+@utils.arg('--merge_metrics', action='store_const',
+           const=True,
+           help='Merge multiple metrics into a single result.')
 def do_measurement_list(mc, args):
     '''List measurements for the specified metric.'''
     fields = {}
-    fields['name'] = args.name
+    if args.name:
+        fields['name'] = args.name
     if args.dimensions:
         fields['dimensions'] = utils.format_parameters(args.dimensions)
     fields['start_time'] = args.starttime
     if args.endtime:
         fields['end_time'] = args.endtime
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+    if args.merge_metrics:
+        fields['merge_metrics'] = args.merge_metrics
+
     try:
         metric = mc.metrics.list_measurements(**fields)
     except exc.HTTPException as he:
@@ -314,6 +336,13 @@ def do_measurement_list(mc, args):
            help='measurements <= UTC time. format: 2014-01-01T00:00:00Z.')
 @utils.arg('--period', metavar='<PERIOD>',
            help='number of seconds per interval (default is 300)')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
+@utils.arg('--merge_metrics', action='store_const',
+           const=True,
+           help='Merge multiple metrics into a single result.')
 def do_metric_statistics(mc, args):
     '''List measurement statistics for the specified metric.'''
     statistic_types = ['AVG', 'MIN', 'MAX', 'COUNT', 'SUM']
@@ -334,6 +363,13 @@ def do_metric_statistics(mc, args):
     if args.period:
         fields['period'] = args.period
     fields['statistics'] = args.statistics
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+    if args.merge_metrics:
+        fields['merge_metrics'] = args.merge_metrics
+
     try:
         metric = mc.metrics.list_statistics(**fields)
     except exc.HTTPException as he:
@@ -438,10 +474,20 @@ def do_notification_show(mc, args):
         utils.print_dict(notification, formatters=formatters)
 
 
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
 def do_notification_list(mc, args):
     '''List notifications for this tenant.'''
+    fields = {}
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
+
     try:
-        notification = mc.notifications.list()
+        notification = mc.notifications.list(**fields)
     except exc.HTTPException as he:
         raise exc.CommandError(
             'HTTPException code=%s message=%s' %
@@ -616,6 +662,10 @@ def do_alarm_definition_show(mc, args):
            'Dimensions need quoting when they contain special chars [&,(,),{,},>,<] '
            'that confuse the CLI parser.',
            action='append')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
 def do_alarm_definition_list(mc, args):
     '''List alarm definitions for this tenant.'''
     fields = {}
@@ -623,6 +673,10 @@ def do_alarm_definition_list(mc, args):
         fields['name'] = args.name
     if args.dimensions:
         fields['dimensions'] = utils.format_parameters(args.dimensions)
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
     try:
         alarm = mc.alarm_definitions.list(**fields)
     except exc.HTTPException as he:
@@ -813,6 +867,10 @@ def do_alarm_definition_patch(mc, args):
            action='append')
 @utils.arg('--state', metavar='<ALARM_STATE>',
            help='ALARM_STATE is one of [UNDETERMINED, OK, ALARM].')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
 def do_alarm_list(mc, args):
     '''List alarms for this tenant.'''
     fields = {}
@@ -829,6 +887,10 @@ def do_alarm_list(mc, args):
             print(errmsg)
             return
         fields['state'] = args.state
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
     try:
         alarm = mc.alarms.list(**fields)
     except exc.HTTPException as he:
@@ -979,10 +1041,18 @@ def output_alarm_history(args, alarm_history):
 
 @utils.arg('id', metavar='<ALARM_ID>',
            help='The ID of the alarm.')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
 def do_alarm_history(mc, args):
     '''Alarm state transition history.'''
     fields = {}
     fields['alarm_id'] = args.id
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
     try:
         alarm = mc.alarms.history(**fields)
     except exc.HTTPException as he:
@@ -1004,6 +1074,10 @@ def do_alarm_history(mc, args):
            help='measurements >= UTC time. format: 2014-01-01T00:00:00Z.')
 @utils.arg('--endtime', metavar='<UTC_END_TIME>',
            help='measurements <= UTC time. format: 2014-01-01T00:00:00Z.')
+@utils.arg('--offset', metavar='<OFFSET LOCATION>',
+           help='The offset used to paginate the return data.')
+@utils.arg('--limit', metavar='<RETURN LIMIT>',
+           help='The amount of data to be returned up to the API maximum limit.')
 def do_alarm_history_list(mc, args):
     '''List alarms state history.'''
     fields = {}
@@ -1013,6 +1087,10 @@ def do_alarm_history_list(mc, args):
         fields['start_time'] = args.starttime
     if args.endtime:
         fields['end_time'] = args.endtime
+    if args.limit:
+        fields['limit'] = args.limit
+    if args.offset:
+        fields['offset'] = args.offset
     try:
         alarm = mc.alarms.history_list(**fields)
     except exc.HTTPException as he:
